/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package v1

import (
	"context"
	"time"

	v1 "github.com/kubeovn/kube-ovn/pkg/apis/kubeovn/v1"
	scheme "github.com/kubeovn/kube-ovn/pkg/client/clientset/versioned/scheme"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// VipsGetter has a method to return a VipInterface.
// A group's client should implement this interface.
type VipsGetter interface {
	Vips() VipInterface
}

// VipInterface has methods to work with Vip resources.
type VipInterface interface {
	Create(ctx context.Context, vip *v1.Vip, opts metav1.CreateOptions) (*v1.Vip, error)
	Update(ctx context.Context, vip *v1.Vip, opts metav1.UpdateOptions) (*v1.Vip, error)
	UpdateStatus(ctx context.Context, vip *v1.Vip, opts metav1.UpdateOptions) (*v1.Vip, error)
	Delete(ctx context.Context, name string, opts metav1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts metav1.DeleteOptions, listOpts metav1.ListOptions) error
	Get(ctx context.Context, name string, opts metav1.GetOptions) (*v1.Vip, error)
	List(ctx context.Context, opts metav1.ListOptions) (*v1.VipList, error)
	Watch(ctx context.Context, opts metav1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts metav1.PatchOptions, subresources ...string) (result *v1.Vip, err error)
	VipExpansion
}

// vips implements VipInterface
type vips struct {
	client rest.Interface
}

// newVips returns a Vips
func newVips(c *KubeovnV1Client) *vips {
	return &vips{
		client: c.RESTClient(),
	}
}

// Get takes name of the vip, and returns the corresponding vip object, and an error if there is any.
func (c *vips) Get(ctx context.Context, name string, options metav1.GetOptions) (result *v1.Vip, err error) {
	result = &v1.Vip{}
	err = c.client.Get().
		Resource("vips").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of Vips that match those selectors.
func (c *vips) List(ctx context.Context, opts metav1.ListOptions) (result *v1.VipList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1.VipList{}
	err = c.client.Get().
		Resource("vips").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested vips.
func (c *vips) Watch(ctx context.Context, opts metav1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Resource("vips").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a vip and creates it.  Returns the server's representation of the vip, and an error, if there is any.
func (c *vips) Create(ctx context.Context, vip *v1.Vip, opts metav1.CreateOptions) (result *v1.Vip, err error) {
	result = &v1.Vip{}
	err = c.client.Post().
		Resource("vips").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(vip).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a vip and updates it. Returns the server's representation of the vip, and an error, if there is any.
func (c *vips) Update(ctx context.Context, vip *v1.Vip, opts metav1.UpdateOptions) (result *v1.Vip, err error) {
	result = &v1.Vip{}
	err = c.client.Put().
		Resource("vips").
		Name(vip.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(vip).
		Do(ctx).
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *vips) UpdateStatus(ctx context.Context, vip *v1.Vip, opts metav1.UpdateOptions) (result *v1.Vip, err error) {
	result = &v1.Vip{}
	err = c.client.Put().
		Resource("vips").
		Name(vip.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(vip).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the vip and deletes it. Returns an error if one occurs.
func (c *vips) Delete(ctx context.Context, name string, opts metav1.DeleteOptions) error {
	return c.client.Delete().
		Resource("vips").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *vips) DeleteCollection(ctx context.Context, opts metav1.DeleteOptions, listOpts metav1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Resource("vips").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched vip.
func (c *vips) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts metav1.PatchOptions, subresources ...string) (result *v1.Vip, err error) {
	result = &v1.Vip{}
	err = c.client.Patch(pt).
		Resource("vips").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
